from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ForceChangeRateUnits(Enum):
        """
            ForceChangeRateUnits enumeration
        """
        
        NewtonPerMinute = 'NewtonPerMinute'
        """
            
        """
        
        NewtonPerSecond = 'NewtonPerSecond'
        """
            
        """
        
        PoundForcePerMinute = 'PoundForcePerMinute'
        """
            
        """
        
        PoundForcePerSecond = 'PoundForcePerSecond'
        """
            
        """
        
        DecanewtonPerMinute = 'DecanewtonPerMinute'
        """
            
        """
        
        KilonewtonPerMinute = 'KilonewtonPerMinute'
        """
            
        """
        
        NanonewtonPerSecond = 'NanonewtonPerSecond'
        """
            
        """
        
        MicronewtonPerSecond = 'MicronewtonPerSecond'
        """
            
        """
        
        MillinewtonPerSecond = 'MillinewtonPerSecond'
        """
            
        """
        
        CentinewtonPerSecond = 'CentinewtonPerSecond'
        """
            
        """
        
        DecinewtonPerSecond = 'DecinewtonPerSecond'
        """
            
        """
        
        DecanewtonPerSecond = 'DecanewtonPerSecond'
        """
            
        """
        
        KilonewtonPerSecond = 'KilonewtonPerSecond'
        """
            
        """
        
        KilopoundForcePerMinute = 'KilopoundForcePerMinute'
        """
            
        """
        
        KilopoundForcePerSecond = 'KilopoundForcePerSecond'
        """
            
        """
        

class ForceChangeRateDto:
    """
    A DTO representation of a ForceChangeRate

    Attributes:
        value (float): The value of the ForceChangeRate.
        unit (ForceChangeRateUnits): The specific unit that the ForceChangeRate value is representing.
    """

    def __init__(self, value: float, unit: ForceChangeRateUnits):
        """
        Create a new DTO representation of a ForceChangeRate

        Parameters:
            value (float): The value of the ForceChangeRate.
            unit (ForceChangeRateUnits): The specific unit that the ForceChangeRate value is representing.
        """
        self.value: float = value
        """
        The value of the ForceChangeRate
        """
        self.unit: ForceChangeRateUnits = unit
        """
        The specific unit that the ForceChangeRate value is representing
        """

    def to_json(self):
        """
        Get a ForceChangeRate DTO JSON object representing the current unit.

        :return: JSON object represents ForceChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ForceChangeRate DTO from a json representation.

        :param data: The ForceChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonPerSecond"}
        :return: A new instance of ForceChangeRateDto.
        :rtype: ForceChangeRateDto
        """
        return ForceChangeRateDto(value=data["value"], unit=ForceChangeRateUnits(data["unit"]))


class ForceChangeRate(AbstractMeasure):
    """
    Force change rate is the ratio of the force change to the time during which the change occurred (value of force changes per unit time).

    Args:
        value (float): The value.
        from_unit (ForceChangeRateUnits): The ForceChangeRate unit to create from, The default unit is NewtonPerSecond
    """
    def __init__(self, value: float, from_unit: ForceChangeRateUnits = ForceChangeRateUnits.NewtonPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__newtons_per_minute = None
        
        self.__newtons_per_second = None
        
        self.__pounds_force_per_minute = None
        
        self.__pounds_force_per_second = None
        
        self.__decanewtons_per_minute = None
        
        self.__kilonewtons_per_minute = None
        
        self.__nanonewtons_per_second = None
        
        self.__micronewtons_per_second = None
        
        self.__millinewtons_per_second = None
        
        self.__centinewtons_per_second = None
        
        self.__decinewtons_per_second = None
        
        self.__decanewtons_per_second = None
        
        self.__kilonewtons_per_second = None
        
        self.__kilopounds_force_per_minute = None
        
        self.__kilopounds_force_per_second = None
        

    def convert(self, unit: ForceChangeRateUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ForceChangeRateUnits = ForceChangeRateUnits.NewtonPerSecond) -> ForceChangeRateDto:
        """
        Get a new instance of ForceChangeRate DTO representing the current unit.

        :param hold_in_unit: The specific ForceChangeRate unit to store the ForceChangeRate value in the DTO representation.
        :type hold_in_unit: ForceChangeRateUnits
        :return: A new instance of ForceChangeRateDto.
        :rtype: ForceChangeRateDto
        """
        return ForceChangeRateDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ForceChangeRateUnits = ForceChangeRateUnits.NewtonPerSecond):
        """
        Get a ForceChangeRate DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ForceChangeRate unit to store the ForceChangeRate value in the DTO representation.
        :type hold_in_unit: ForceChangeRateUnits
        :return: JSON object represents ForceChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(force_change_rate_dto: ForceChangeRateDto):
        """
        Obtain a new instance of ForceChangeRate from a DTO unit object.

        :param force_change_rate_dto: The ForceChangeRate DTO representation.
        :type force_change_rate_dto: ForceChangeRateDto
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(force_change_rate_dto.value, force_change_rate_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ForceChangeRate from a DTO unit json representation.

        :param data: The ForceChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonPerSecond"}
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate.from_dto(ForceChangeRateDto.from_json(data))

    def __convert_from_base(self, from_unit: ForceChangeRateUnits) -> float:
        value = self._value
        
        if from_unit == ForceChangeRateUnits.NewtonPerMinute:
            return (value * 60)
        
        if from_unit == ForceChangeRateUnits.NewtonPerSecond:
            return (value)
        
        if from_unit == ForceChangeRateUnits.PoundForcePerMinute:
            return (value / 4.4482216152605095551842641431421 * 60)
        
        if from_unit == ForceChangeRateUnits.PoundForcePerSecond:
            return (value / 4.4482216152605095551842641431421)
        
        if from_unit == ForceChangeRateUnits.DecanewtonPerMinute:
            return ((value * 60) / 10.0)
        
        if from_unit == ForceChangeRateUnits.KilonewtonPerMinute:
            return ((value * 60) / 1000.0)
        
        if from_unit == ForceChangeRateUnits.NanonewtonPerSecond:
            return ((value) / 1e-09)
        
        if from_unit == ForceChangeRateUnits.MicronewtonPerSecond:
            return ((value) / 1e-06)
        
        if from_unit == ForceChangeRateUnits.MillinewtonPerSecond:
            return ((value) / 0.001)
        
        if from_unit == ForceChangeRateUnits.CentinewtonPerSecond:
            return ((value) / 0.01)
        
        if from_unit == ForceChangeRateUnits.DecinewtonPerSecond:
            return ((value) / 0.1)
        
        if from_unit == ForceChangeRateUnits.DecanewtonPerSecond:
            return ((value) / 10.0)
        
        if from_unit == ForceChangeRateUnits.KilonewtonPerSecond:
            return ((value) / 1000.0)
        
        if from_unit == ForceChangeRateUnits.KilopoundForcePerMinute:
            return ((value / 4.4482216152605095551842641431421 * 60) / 1000.0)
        
        if from_unit == ForceChangeRateUnits.KilopoundForcePerSecond:
            return ((value / 4.4482216152605095551842641431421) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ForceChangeRateUnits) -> float:
        
        if to_unit == ForceChangeRateUnits.NewtonPerMinute:
            return (value / 60)
        
        if to_unit == ForceChangeRateUnits.NewtonPerSecond:
            return (value)
        
        if to_unit == ForceChangeRateUnits.PoundForcePerMinute:
            return (value * 4.4482216152605095551842641431421 / 60)
        
        if to_unit == ForceChangeRateUnits.PoundForcePerSecond:
            return (value * 4.4482216152605095551842641431421)
        
        if to_unit == ForceChangeRateUnits.DecanewtonPerMinute:
            return ((value / 60) * 10.0)
        
        if to_unit == ForceChangeRateUnits.KilonewtonPerMinute:
            return ((value / 60) * 1000.0)
        
        if to_unit == ForceChangeRateUnits.NanonewtonPerSecond:
            return ((value) * 1e-09)
        
        if to_unit == ForceChangeRateUnits.MicronewtonPerSecond:
            return ((value) * 1e-06)
        
        if to_unit == ForceChangeRateUnits.MillinewtonPerSecond:
            return ((value) * 0.001)
        
        if to_unit == ForceChangeRateUnits.CentinewtonPerSecond:
            return ((value) * 0.01)
        
        if to_unit == ForceChangeRateUnits.DecinewtonPerSecond:
            return ((value) * 0.1)
        
        if to_unit == ForceChangeRateUnits.DecanewtonPerSecond:
            return ((value) * 10.0)
        
        if to_unit == ForceChangeRateUnits.KilonewtonPerSecond:
            return ((value) * 1000.0)
        
        if to_unit == ForceChangeRateUnits.KilopoundForcePerMinute:
            return ((value * 4.4482216152605095551842641431421 / 60) * 1000.0)
        
        if to_unit == ForceChangeRateUnits.KilopoundForcePerSecond:
            return ((value * 4.4482216152605095551842641431421) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_newtons_per_minute(newtons_per_minute: float):
        """
        Create a new instance of ForceChangeRate from a value in newtons_per_minute.

        

        :param meters: The ForceChangeRate value in newtons_per_minute.
        :type newtons_per_minute: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(newtons_per_minute, ForceChangeRateUnits.NewtonPerMinute)

    
    @staticmethod
    def from_newtons_per_second(newtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in newtons_per_second.

        

        :param meters: The ForceChangeRate value in newtons_per_second.
        :type newtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(newtons_per_second, ForceChangeRateUnits.NewtonPerSecond)

    
    @staticmethod
    def from_pounds_force_per_minute(pounds_force_per_minute: float):
        """
        Create a new instance of ForceChangeRate from a value in pounds_force_per_minute.

        

        :param meters: The ForceChangeRate value in pounds_force_per_minute.
        :type pounds_force_per_minute: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(pounds_force_per_minute, ForceChangeRateUnits.PoundForcePerMinute)

    
    @staticmethod
    def from_pounds_force_per_second(pounds_force_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in pounds_force_per_second.

        

        :param meters: The ForceChangeRate value in pounds_force_per_second.
        :type pounds_force_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(pounds_force_per_second, ForceChangeRateUnits.PoundForcePerSecond)

    
    @staticmethod
    def from_decanewtons_per_minute(decanewtons_per_minute: float):
        """
        Create a new instance of ForceChangeRate from a value in decanewtons_per_minute.

        

        :param meters: The ForceChangeRate value in decanewtons_per_minute.
        :type decanewtons_per_minute: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(decanewtons_per_minute, ForceChangeRateUnits.DecanewtonPerMinute)

    
    @staticmethod
    def from_kilonewtons_per_minute(kilonewtons_per_minute: float):
        """
        Create a new instance of ForceChangeRate from a value in kilonewtons_per_minute.

        

        :param meters: The ForceChangeRate value in kilonewtons_per_minute.
        :type kilonewtons_per_minute: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(kilonewtons_per_minute, ForceChangeRateUnits.KilonewtonPerMinute)

    
    @staticmethod
    def from_nanonewtons_per_second(nanonewtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in nanonewtons_per_second.

        

        :param meters: The ForceChangeRate value in nanonewtons_per_second.
        :type nanonewtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(nanonewtons_per_second, ForceChangeRateUnits.NanonewtonPerSecond)

    
    @staticmethod
    def from_micronewtons_per_second(micronewtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in micronewtons_per_second.

        

        :param meters: The ForceChangeRate value in micronewtons_per_second.
        :type micronewtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(micronewtons_per_second, ForceChangeRateUnits.MicronewtonPerSecond)

    
    @staticmethod
    def from_millinewtons_per_second(millinewtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in millinewtons_per_second.

        

        :param meters: The ForceChangeRate value in millinewtons_per_second.
        :type millinewtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(millinewtons_per_second, ForceChangeRateUnits.MillinewtonPerSecond)

    
    @staticmethod
    def from_centinewtons_per_second(centinewtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in centinewtons_per_second.

        

        :param meters: The ForceChangeRate value in centinewtons_per_second.
        :type centinewtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(centinewtons_per_second, ForceChangeRateUnits.CentinewtonPerSecond)

    
    @staticmethod
    def from_decinewtons_per_second(decinewtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in decinewtons_per_second.

        

        :param meters: The ForceChangeRate value in decinewtons_per_second.
        :type decinewtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(decinewtons_per_second, ForceChangeRateUnits.DecinewtonPerSecond)

    
    @staticmethod
    def from_decanewtons_per_second(decanewtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in decanewtons_per_second.

        

        :param meters: The ForceChangeRate value in decanewtons_per_second.
        :type decanewtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(decanewtons_per_second, ForceChangeRateUnits.DecanewtonPerSecond)

    
    @staticmethod
    def from_kilonewtons_per_second(kilonewtons_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in kilonewtons_per_second.

        

        :param meters: The ForceChangeRate value in kilonewtons_per_second.
        :type kilonewtons_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(kilonewtons_per_second, ForceChangeRateUnits.KilonewtonPerSecond)

    
    @staticmethod
    def from_kilopounds_force_per_minute(kilopounds_force_per_minute: float):
        """
        Create a new instance of ForceChangeRate from a value in kilopounds_force_per_minute.

        

        :param meters: The ForceChangeRate value in kilopounds_force_per_minute.
        :type kilopounds_force_per_minute: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(kilopounds_force_per_minute, ForceChangeRateUnits.KilopoundForcePerMinute)

    
    @staticmethod
    def from_kilopounds_force_per_second(kilopounds_force_per_second: float):
        """
        Create a new instance of ForceChangeRate from a value in kilopounds_force_per_second.

        

        :param meters: The ForceChangeRate value in kilopounds_force_per_second.
        :type kilopounds_force_per_second: float
        :return: A new instance of ForceChangeRate.
        :rtype: ForceChangeRate
        """
        return ForceChangeRate(kilopounds_force_per_second, ForceChangeRateUnits.KilopoundForcePerSecond)

    
    @property
    def newtons_per_minute(self) -> float:
        """
        
        """
        if self.__newtons_per_minute != None:
            return self.__newtons_per_minute
        self.__newtons_per_minute = self.__convert_from_base(ForceChangeRateUnits.NewtonPerMinute)
        return self.__newtons_per_minute

    
    @property
    def newtons_per_second(self) -> float:
        """
        
        """
        if self.__newtons_per_second != None:
            return self.__newtons_per_second
        self.__newtons_per_second = self.__convert_from_base(ForceChangeRateUnits.NewtonPerSecond)
        return self.__newtons_per_second

    
    @property
    def pounds_force_per_minute(self) -> float:
        """
        
        """
        if self.__pounds_force_per_minute != None:
            return self.__pounds_force_per_minute
        self.__pounds_force_per_minute = self.__convert_from_base(ForceChangeRateUnits.PoundForcePerMinute)
        return self.__pounds_force_per_minute

    
    @property
    def pounds_force_per_second(self) -> float:
        """
        
        """
        if self.__pounds_force_per_second != None:
            return self.__pounds_force_per_second
        self.__pounds_force_per_second = self.__convert_from_base(ForceChangeRateUnits.PoundForcePerSecond)
        return self.__pounds_force_per_second

    
    @property
    def decanewtons_per_minute(self) -> float:
        """
        
        """
        if self.__decanewtons_per_minute != None:
            return self.__decanewtons_per_minute
        self.__decanewtons_per_minute = self.__convert_from_base(ForceChangeRateUnits.DecanewtonPerMinute)
        return self.__decanewtons_per_minute

    
    @property
    def kilonewtons_per_minute(self) -> float:
        """
        
        """
        if self.__kilonewtons_per_minute != None:
            return self.__kilonewtons_per_minute
        self.__kilonewtons_per_minute = self.__convert_from_base(ForceChangeRateUnits.KilonewtonPerMinute)
        return self.__kilonewtons_per_minute

    
    @property
    def nanonewtons_per_second(self) -> float:
        """
        
        """
        if self.__nanonewtons_per_second != None:
            return self.__nanonewtons_per_second
        self.__nanonewtons_per_second = self.__convert_from_base(ForceChangeRateUnits.NanonewtonPerSecond)
        return self.__nanonewtons_per_second

    
    @property
    def micronewtons_per_second(self) -> float:
        """
        
        """
        if self.__micronewtons_per_second != None:
            return self.__micronewtons_per_second
        self.__micronewtons_per_second = self.__convert_from_base(ForceChangeRateUnits.MicronewtonPerSecond)
        return self.__micronewtons_per_second

    
    @property
    def millinewtons_per_second(self) -> float:
        """
        
        """
        if self.__millinewtons_per_second != None:
            return self.__millinewtons_per_second
        self.__millinewtons_per_second = self.__convert_from_base(ForceChangeRateUnits.MillinewtonPerSecond)
        return self.__millinewtons_per_second

    
    @property
    def centinewtons_per_second(self) -> float:
        """
        
        """
        if self.__centinewtons_per_second != None:
            return self.__centinewtons_per_second
        self.__centinewtons_per_second = self.__convert_from_base(ForceChangeRateUnits.CentinewtonPerSecond)
        return self.__centinewtons_per_second

    
    @property
    def decinewtons_per_second(self) -> float:
        """
        
        """
        if self.__decinewtons_per_second != None:
            return self.__decinewtons_per_second
        self.__decinewtons_per_second = self.__convert_from_base(ForceChangeRateUnits.DecinewtonPerSecond)
        return self.__decinewtons_per_second

    
    @property
    def decanewtons_per_second(self) -> float:
        """
        
        """
        if self.__decanewtons_per_second != None:
            return self.__decanewtons_per_second
        self.__decanewtons_per_second = self.__convert_from_base(ForceChangeRateUnits.DecanewtonPerSecond)
        return self.__decanewtons_per_second

    
    @property
    def kilonewtons_per_second(self) -> float:
        """
        
        """
        if self.__kilonewtons_per_second != None:
            return self.__kilonewtons_per_second
        self.__kilonewtons_per_second = self.__convert_from_base(ForceChangeRateUnits.KilonewtonPerSecond)
        return self.__kilonewtons_per_second

    
    @property
    def kilopounds_force_per_minute(self) -> float:
        """
        
        """
        if self.__kilopounds_force_per_minute != None:
            return self.__kilopounds_force_per_minute
        self.__kilopounds_force_per_minute = self.__convert_from_base(ForceChangeRateUnits.KilopoundForcePerMinute)
        return self.__kilopounds_force_per_minute

    
    @property
    def kilopounds_force_per_second(self) -> float:
        """
        
        """
        if self.__kilopounds_force_per_second != None:
            return self.__kilopounds_force_per_second
        self.__kilopounds_force_per_second = self.__convert_from_base(ForceChangeRateUnits.KilopoundForcePerSecond)
        return self.__kilopounds_force_per_second

    
    def to_string(self, unit: ForceChangeRateUnits = ForceChangeRateUnits.NewtonPerSecond, fractional_digits: int = None) -> str:
        """
        Format the ForceChangeRate to a string.
        
        Note: the default format for ForceChangeRate is NewtonPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ForceChangeRate. Default is 'NewtonPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ForceChangeRateUnits.NewtonPerMinute:
            return f"""{super()._truncate_fraction_digits(self.newtons_per_minute, fractional_digits)} N/min"""
        
        if unit == ForceChangeRateUnits.NewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.newtons_per_second, fractional_digits)} N/s"""
        
        if unit == ForceChangeRateUnits.PoundForcePerMinute:
            return f"""{super()._truncate_fraction_digits(self.pounds_force_per_minute, fractional_digits)} lbf/min"""
        
        if unit == ForceChangeRateUnits.PoundForcePerSecond:
            return f"""{super()._truncate_fraction_digits(self.pounds_force_per_second, fractional_digits)} lbf/s"""
        
        if unit == ForceChangeRateUnits.DecanewtonPerMinute:
            return f"""{super()._truncate_fraction_digits(self.decanewtons_per_minute, fractional_digits)} daN/min"""
        
        if unit == ForceChangeRateUnits.KilonewtonPerMinute:
            return f"""{super()._truncate_fraction_digits(self.kilonewtons_per_minute, fractional_digits)} kN/min"""
        
        if unit == ForceChangeRateUnits.NanonewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.nanonewtons_per_second, fractional_digits)} nN/s"""
        
        if unit == ForceChangeRateUnits.MicronewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.micronewtons_per_second, fractional_digits)} μN/s"""
        
        if unit == ForceChangeRateUnits.MillinewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.millinewtons_per_second, fractional_digits)} mN/s"""
        
        if unit == ForceChangeRateUnits.CentinewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.centinewtons_per_second, fractional_digits)} cN/s"""
        
        if unit == ForceChangeRateUnits.DecinewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.decinewtons_per_second, fractional_digits)} dN/s"""
        
        if unit == ForceChangeRateUnits.DecanewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.decanewtons_per_second, fractional_digits)} daN/s"""
        
        if unit == ForceChangeRateUnits.KilonewtonPerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilonewtons_per_second, fractional_digits)} kN/s"""
        
        if unit == ForceChangeRateUnits.KilopoundForcePerMinute:
            return f"""{super()._truncate_fraction_digits(self.kilopounds_force_per_minute, fractional_digits)} klbf/min"""
        
        if unit == ForceChangeRateUnits.KilopoundForcePerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilopounds_force_per_second, fractional_digits)} klbf/s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ForceChangeRateUnits = ForceChangeRateUnits.NewtonPerSecond) -> str:
        """
        Get ForceChangeRate unit abbreviation.
        Note! the default abbreviation for ForceChangeRate is NewtonPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ForceChangeRateUnits.NewtonPerMinute:
            return """N/min"""
        
        if unit_abbreviation == ForceChangeRateUnits.NewtonPerSecond:
            return """N/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.PoundForcePerMinute:
            return """lbf/min"""
        
        if unit_abbreviation == ForceChangeRateUnits.PoundForcePerSecond:
            return """lbf/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.DecanewtonPerMinute:
            return """daN/min"""
        
        if unit_abbreviation == ForceChangeRateUnits.KilonewtonPerMinute:
            return """kN/min"""
        
        if unit_abbreviation == ForceChangeRateUnits.NanonewtonPerSecond:
            return """nN/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.MicronewtonPerSecond:
            return """μN/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.MillinewtonPerSecond:
            return """mN/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.CentinewtonPerSecond:
            return """cN/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.DecinewtonPerSecond:
            return """dN/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.DecanewtonPerSecond:
            return """daN/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.KilonewtonPerSecond:
            return """kN/s"""
        
        if unit_abbreviation == ForceChangeRateUnits.KilopoundForcePerMinute:
            return """klbf/min"""
        
        if unit_abbreviation == ForceChangeRateUnits.KilopoundForcePerSecond:
            return """klbf/s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricConductanceUnits(Enum):
        """
            ElectricConductanceUnits enumeration
        """
        
        Siemens = 'Siemens'
        """
            
        """
        
        Nanosiemens = 'Nanosiemens'
        """
            
        """
        
        Microsiemens = 'Microsiemens'
        """
            
        """
        
        Millisiemens = 'Millisiemens'
        """
            
        """
        
        Kilosiemens = 'Kilosiemens'
        """
            
        """
        

class ElectricConductanceDto:
    """
    A DTO representation of a ElectricConductance

    Attributes:
        value (float): The value of the ElectricConductance.
        unit (ElectricConductanceUnits): The specific unit that the ElectricConductance value is representing.
    """

    def __init__(self, value: float, unit: ElectricConductanceUnits):
        """
        Create a new DTO representation of a ElectricConductance

        Parameters:
            value (float): The value of the ElectricConductance.
            unit (ElectricConductanceUnits): The specific unit that the ElectricConductance value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricConductance
        """
        self.unit: ElectricConductanceUnits = unit
        """
        The specific unit that the ElectricConductance value is representing
        """

    def to_json(self):
        """
        Get a ElectricConductance DTO JSON object representing the current unit.

        :return: JSON object represents ElectricConductance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Siemens"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricConductance DTO from a json representation.

        :param data: The ElectricConductance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Siemens"}
        :return: A new instance of ElectricConductanceDto.
        :rtype: ElectricConductanceDto
        """
        return ElectricConductanceDto(value=data["value"], unit=ElectricConductanceUnits(data["unit"]))


class ElectricConductance(AbstractMeasure):
    """
    The electrical conductance of an electrical conductor is a measure of the easeness to pass an electric current through that conductor.

    Args:
        value (float): The value.
        from_unit (ElectricConductanceUnits): The ElectricConductance unit to create from, The default unit is Siemens
    """
    def __init__(self, value: float, from_unit: ElectricConductanceUnits = ElectricConductanceUnits.Siemens):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__siemens = None
        
        self.__nanosiemens = None
        
        self.__microsiemens = None
        
        self.__millisiemens = None
        
        self.__kilosiemens = None
        

    def convert(self, unit: ElectricConductanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricConductanceUnits = ElectricConductanceUnits.Siemens) -> ElectricConductanceDto:
        """
        Get a new instance of ElectricConductance DTO representing the current unit.

        :param hold_in_unit: The specific ElectricConductance unit to store the ElectricConductance value in the DTO representation.
        :type hold_in_unit: ElectricConductanceUnits
        :return: A new instance of ElectricConductanceDto.
        :rtype: ElectricConductanceDto
        """
        return ElectricConductanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricConductanceUnits = ElectricConductanceUnits.Siemens):
        """
        Get a ElectricConductance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricConductance unit to store the ElectricConductance value in the DTO representation.
        :type hold_in_unit: ElectricConductanceUnits
        :return: JSON object represents ElectricConductance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Siemens"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_conductance_dto: ElectricConductanceDto):
        """
        Obtain a new instance of ElectricConductance from a DTO unit object.

        :param electric_conductance_dto: The ElectricConductance DTO representation.
        :type electric_conductance_dto: ElectricConductanceDto
        :return: A new instance of ElectricConductance.
        :rtype: ElectricConductance
        """
        return ElectricConductance(electric_conductance_dto.value, electric_conductance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricConductance from a DTO unit json representation.

        :param data: The ElectricConductance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Siemens"}
        :return: A new instance of ElectricConductance.
        :rtype: ElectricConductance
        """
        return ElectricConductance.from_dto(ElectricConductanceDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricConductanceUnits) -> float:
        value = self._value
        
        if from_unit == ElectricConductanceUnits.Siemens:
            return (value)
        
        if from_unit == ElectricConductanceUnits.Nanosiemens:
            return ((value) / 1e-09)
        
        if from_unit == ElectricConductanceUnits.Microsiemens:
            return ((value) / 1e-06)
        
        if from_unit == ElectricConductanceUnits.Millisiemens:
            return ((value) / 0.001)
        
        if from_unit == ElectricConductanceUnits.Kilosiemens:
            return ((value) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricConductanceUnits) -> float:
        
        if to_unit == ElectricConductanceUnits.Siemens:
            return (value)
        
        if to_unit == ElectricConductanceUnits.Nanosiemens:
            return ((value) * 1e-09)
        
        if to_unit == ElectricConductanceUnits.Microsiemens:
            return ((value) * 1e-06)
        
        if to_unit == ElectricConductanceUnits.Millisiemens:
            return ((value) * 0.001)
        
        if to_unit == ElectricConductanceUnits.Kilosiemens:
            return ((value) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_siemens(siemens: float):
        """
        Create a new instance of ElectricConductance from a value in siemens.

        

        :param meters: The ElectricConductance value in siemens.
        :type siemens: float
        :return: A new instance of ElectricConductance.
        :rtype: ElectricConductance
        """
        return ElectricConductance(siemens, ElectricConductanceUnits.Siemens)

    
    @staticmethod
    def from_nanosiemens(nanosiemens: float):
        """
        Create a new instance of ElectricConductance from a value in nanosiemens.

        

        :param meters: The ElectricConductance value in nanosiemens.
        :type nanosiemens: float
        :return: A new instance of ElectricConductance.
        :rtype: ElectricConductance
        """
        return ElectricConductance(nanosiemens, ElectricConductanceUnits.Nanosiemens)

    
    @staticmethod
    def from_microsiemens(microsiemens: float):
        """
        Create a new instance of ElectricConductance from a value in microsiemens.

        

        :param meters: The ElectricConductance value in microsiemens.
        :type microsiemens: float
        :return: A new instance of ElectricConductance.
        :rtype: ElectricConductance
        """
        return ElectricConductance(microsiemens, ElectricConductanceUnits.Microsiemens)

    
    @staticmethod
    def from_millisiemens(millisiemens: float):
        """
        Create a new instance of ElectricConductance from a value in millisiemens.

        

        :param meters: The ElectricConductance value in millisiemens.
        :type millisiemens: float
        :return: A new instance of ElectricConductance.
        :rtype: ElectricConductance
        """
        return ElectricConductance(millisiemens, ElectricConductanceUnits.Millisiemens)

    
    @staticmethod
    def from_kilosiemens(kilosiemens: float):
        """
        Create a new instance of ElectricConductance from a value in kilosiemens.

        

        :param meters: The ElectricConductance value in kilosiemens.
        :type kilosiemens: float
        :return: A new instance of ElectricConductance.
        :rtype: ElectricConductance
        """
        return ElectricConductance(kilosiemens, ElectricConductanceUnits.Kilosiemens)

    
    @property
    def siemens(self) -> float:
        """
        
        """
        if self.__siemens != None:
            return self.__siemens
        self.__siemens = self.__convert_from_base(ElectricConductanceUnits.Siemens)
        return self.__siemens

    
    @property
    def nanosiemens(self) -> float:
        """
        
        """
        if self.__nanosiemens != None:
            return self.__nanosiemens
        self.__nanosiemens = self.__convert_from_base(ElectricConductanceUnits.Nanosiemens)
        return self.__nanosiemens

    
    @property
    def microsiemens(self) -> float:
        """
        
        """
        if self.__microsiemens != None:
            return self.__microsiemens
        self.__microsiemens = self.__convert_from_base(ElectricConductanceUnits.Microsiemens)
        return self.__microsiemens

    
    @property
    def millisiemens(self) -> float:
        """
        
        """
        if self.__millisiemens != None:
            return self.__millisiemens
        self.__millisiemens = self.__convert_from_base(ElectricConductanceUnits.Millisiemens)
        return self.__millisiemens

    
    @property
    def kilosiemens(self) -> float:
        """
        
        """
        if self.__kilosiemens != None:
            return self.__kilosiemens
        self.__kilosiemens = self.__convert_from_base(ElectricConductanceUnits.Kilosiemens)
        return self.__kilosiemens

    
    def to_string(self, unit: ElectricConductanceUnits = ElectricConductanceUnits.Siemens, fractional_digits: int = None) -> str:
        """
        Format the ElectricConductance to a string.
        
        Note: the default format for ElectricConductance is Siemens.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricConductance. Default is 'Siemens'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricConductanceUnits.Siemens:
            return f"""{super()._truncate_fraction_digits(self.siemens, fractional_digits)} S"""
        
        if unit == ElectricConductanceUnits.Nanosiemens:
            return f"""{super()._truncate_fraction_digits(self.nanosiemens, fractional_digits)} nS"""
        
        if unit == ElectricConductanceUnits.Microsiemens:
            return f"""{super()._truncate_fraction_digits(self.microsiemens, fractional_digits)} μS"""
        
        if unit == ElectricConductanceUnits.Millisiemens:
            return f"""{super()._truncate_fraction_digits(self.millisiemens, fractional_digits)} mS"""
        
        if unit == ElectricConductanceUnits.Kilosiemens:
            return f"""{super()._truncate_fraction_digits(self.kilosiemens, fractional_digits)} kS"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricConductanceUnits = ElectricConductanceUnits.Siemens) -> str:
        """
        Get ElectricConductance unit abbreviation.
        Note! the default abbreviation for ElectricConductance is Siemens.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricConductanceUnits.Siemens:
            return """S"""
        
        if unit_abbreviation == ElectricConductanceUnits.Nanosiemens:
            return """nS"""
        
        if unit_abbreviation == ElectricConductanceUnits.Microsiemens:
            return """μS"""
        
        if unit_abbreviation == ElectricConductanceUnits.Millisiemens:
            return """mS"""
        
        if unit_abbreviation == ElectricConductanceUnits.Kilosiemens:
            return """kS"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class FuelEfficiencyUnits(Enum):
        """
            FuelEfficiencyUnits enumeration
        """
        
        LiterPer100Kilometers = 'LiterPer100Kilometers'
        """
            
        """
        
        MilePerUsGallon = 'MilePerUsGallon'
        """
            
        """
        
        MilePerUkGallon = 'MilePerUkGallon'
        """
            
        """
        
        KilometerPerLiter = 'KilometerPerLiter'
        """
            
        """
        

class FuelEfficiencyDto:
    """
    A DTO representation of a FuelEfficiency

    Attributes:
        value (float): The value of the FuelEfficiency.
        unit (FuelEfficiencyUnits): The specific unit that the FuelEfficiency value is representing.
    """

    def __init__(self, value: float, unit: FuelEfficiencyUnits):
        """
        Create a new DTO representation of a FuelEfficiency

        Parameters:
            value (float): The value of the FuelEfficiency.
            unit (FuelEfficiencyUnits): The specific unit that the FuelEfficiency value is representing.
        """
        self.value: float = value
        """
        The value of the FuelEfficiency
        """
        self.unit: FuelEfficiencyUnits = unit
        """
        The specific unit that the FuelEfficiency value is representing
        """

    def to_json(self):
        """
        Get a FuelEfficiency DTO JSON object representing the current unit.

        :return: JSON object represents FuelEfficiency DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "LiterPer100Kilometers"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of FuelEfficiency DTO from a json representation.

        :param data: The FuelEfficiency DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "LiterPer100Kilometers"}
        :return: A new instance of FuelEfficiencyDto.
        :rtype: FuelEfficiencyDto
        """
        return FuelEfficiencyDto(value=data["value"], unit=FuelEfficiencyUnits(data["unit"]))


class FuelEfficiency(AbstractMeasure):
    """
    Fuel efficiency is a form of thermal efficiency, meaning the ratio from effort to result of a process that converts chemical potential energy contained in a carrier (fuel) into kinetic energy or work. Fuel economy is stated as "fuel consumption" in liters per 100 kilometers (L/100 km). In countries using non-metric system, fuel economy is expressed in miles per gallon (mpg) (imperial galon or US galon).

    Args:
        value (float): The value.
        from_unit (FuelEfficiencyUnits): The FuelEfficiency unit to create from, The default unit is LiterPer100Kilometers
    """
    def __init__(self, value: float, from_unit: FuelEfficiencyUnits = FuelEfficiencyUnits.LiterPer100Kilometers):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__liters_per100_kilometers = None
        
        self.__miles_per_us_gallon = None
        
        self.__miles_per_uk_gallon = None
        
        self.__kilometers_per_liters = None
        

    def convert(self, unit: FuelEfficiencyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: FuelEfficiencyUnits = FuelEfficiencyUnits.LiterPer100Kilometers) -> FuelEfficiencyDto:
        """
        Get a new instance of FuelEfficiency DTO representing the current unit.

        :param hold_in_unit: The specific FuelEfficiency unit to store the FuelEfficiency value in the DTO representation.
        :type hold_in_unit: FuelEfficiencyUnits
        :return: A new instance of FuelEfficiencyDto.
        :rtype: FuelEfficiencyDto
        """
        return FuelEfficiencyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: FuelEfficiencyUnits = FuelEfficiencyUnits.LiterPer100Kilometers):
        """
        Get a FuelEfficiency DTO JSON object representing the current unit.

        :param hold_in_unit: The specific FuelEfficiency unit to store the FuelEfficiency value in the DTO representation.
        :type hold_in_unit: FuelEfficiencyUnits
        :return: JSON object represents FuelEfficiency DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "LiterPer100Kilometers"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(fuel_efficiency_dto: FuelEfficiencyDto):
        """
        Obtain a new instance of FuelEfficiency from a DTO unit object.

        :param fuel_efficiency_dto: The FuelEfficiency DTO representation.
        :type fuel_efficiency_dto: FuelEfficiencyDto
        :return: A new instance of FuelEfficiency.
        :rtype: FuelEfficiency
        """
        return FuelEfficiency(fuel_efficiency_dto.value, fuel_efficiency_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of FuelEfficiency from a DTO unit json representation.

        :param data: The FuelEfficiency DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "LiterPer100Kilometers"}
        :return: A new instance of FuelEfficiency.
        :rtype: FuelEfficiency
        """
        return FuelEfficiency.from_dto(FuelEfficiencyDto.from_json(data))

    def __convert_from_base(self, from_unit: FuelEfficiencyUnits) -> float:
        value = self._value
        
        if from_unit == FuelEfficiencyUnits.LiterPer100Kilometers:
            return (value)
        
        if from_unit == FuelEfficiencyUnits.MilePerUsGallon:
            return ((100 * 3.785411784) / (1.609344 * value))
        
        if from_unit == FuelEfficiencyUnits.MilePerUkGallon:
            return ((100 * 4.54609188) / (1.609344 * value))
        
        if from_unit == FuelEfficiencyUnits.KilometerPerLiter:
            return (100 / value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: FuelEfficiencyUnits) -> float:
        
        if to_unit == FuelEfficiencyUnits.LiterPer100Kilometers:
            return (value)
        
        if to_unit == FuelEfficiencyUnits.MilePerUsGallon:
            return ((100 * 3.785411784) / (1.609344 * value))
        
        if to_unit == FuelEfficiencyUnits.MilePerUkGallon:
            return ((100 * 4.54609188) / (1.609344 * value))
        
        if to_unit == FuelEfficiencyUnits.KilometerPerLiter:
            return (100 / value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_liters_per100_kilometers(liters_per100_kilometers: float):
        """
        Create a new instance of FuelEfficiency from a value in liters_per100_kilometers.

        

        :param meters: The FuelEfficiency value in liters_per100_kilometers.
        :type liters_per100_kilometers: float
        :return: A new instance of FuelEfficiency.
        :rtype: FuelEfficiency
        """
        return FuelEfficiency(liters_per100_kilometers, FuelEfficiencyUnits.LiterPer100Kilometers)

    
    @staticmethod
    def from_miles_per_us_gallon(miles_per_us_gallon: float):
        """
        Create a new instance of FuelEfficiency from a value in miles_per_us_gallon.

        

        :param meters: The FuelEfficiency value in miles_per_us_gallon.
        :type miles_per_us_gallon: float
        :return: A new instance of FuelEfficiency.
        :rtype: FuelEfficiency
        """
        return FuelEfficiency(miles_per_us_gallon, FuelEfficiencyUnits.MilePerUsGallon)

    
    @staticmethod
    def from_miles_per_uk_gallon(miles_per_uk_gallon: float):
        """
        Create a new instance of FuelEfficiency from a value in miles_per_uk_gallon.

        

        :param meters: The FuelEfficiency value in miles_per_uk_gallon.
        :type miles_per_uk_gallon: float
        :return: A new instance of FuelEfficiency.
        :rtype: FuelEfficiency
        """
        return FuelEfficiency(miles_per_uk_gallon, FuelEfficiencyUnits.MilePerUkGallon)

    
    @staticmethod
    def from_kilometers_per_liters(kilometers_per_liters: float):
        """
        Create a new instance of FuelEfficiency from a value in kilometers_per_liters.

        

        :param meters: The FuelEfficiency value in kilometers_per_liters.
        :type kilometers_per_liters: float
        :return: A new instance of FuelEfficiency.
        :rtype: FuelEfficiency
        """
        return FuelEfficiency(kilometers_per_liters, FuelEfficiencyUnits.KilometerPerLiter)

    
    @property
    def liters_per100_kilometers(self) -> float:
        """
        
        """
        if self.__liters_per100_kilometers != None:
            return self.__liters_per100_kilometers
        self.__liters_per100_kilometers = self.__convert_from_base(FuelEfficiencyUnits.LiterPer100Kilometers)
        return self.__liters_per100_kilometers

    
    @property
    def miles_per_us_gallon(self) -> float:
        """
        
        """
        if self.__miles_per_us_gallon != None:
            return self.__miles_per_us_gallon
        self.__miles_per_us_gallon = self.__convert_from_base(FuelEfficiencyUnits.MilePerUsGallon)
        return self.__miles_per_us_gallon

    
    @property
    def miles_per_uk_gallon(self) -> float:
        """
        
        """
        if self.__miles_per_uk_gallon != None:
            return self.__miles_per_uk_gallon
        self.__miles_per_uk_gallon = self.__convert_from_base(FuelEfficiencyUnits.MilePerUkGallon)
        return self.__miles_per_uk_gallon

    
    @property
    def kilometers_per_liters(self) -> float:
        """
        
        """
        if self.__kilometers_per_liters != None:
            return self.__kilometers_per_liters
        self.__kilometers_per_liters = self.__convert_from_base(FuelEfficiencyUnits.KilometerPerLiter)
        return self.__kilometers_per_liters

    
    def to_string(self, unit: FuelEfficiencyUnits = FuelEfficiencyUnits.LiterPer100Kilometers, fractional_digits: int = None) -> str:
        """
        Format the FuelEfficiency to a string.
        
        Note: the default format for FuelEfficiency is LiterPer100Kilometers.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the FuelEfficiency. Default is 'LiterPer100Kilometers'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == FuelEfficiencyUnits.LiterPer100Kilometers:
            return f"""{super()._truncate_fraction_digits(self.liters_per100_kilometers, fractional_digits)} L/100km"""
        
        if unit == FuelEfficiencyUnits.MilePerUsGallon:
            return f"""{super()._truncate_fraction_digits(self.miles_per_us_gallon, fractional_digits)} mpg (U.S.)"""
        
        if unit == FuelEfficiencyUnits.MilePerUkGallon:
            return f"""{super()._truncate_fraction_digits(self.miles_per_uk_gallon, fractional_digits)} mpg (imp.)"""
        
        if unit == FuelEfficiencyUnits.KilometerPerLiter:
            return f"""{super()._truncate_fraction_digits(self.kilometers_per_liters, fractional_digits)} km/L"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: FuelEfficiencyUnits = FuelEfficiencyUnits.LiterPer100Kilometers) -> str:
        """
        Get FuelEfficiency unit abbreviation.
        Note! the default abbreviation for FuelEfficiency is LiterPer100Kilometers.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == FuelEfficiencyUnits.LiterPer100Kilometers:
            return """L/100km"""
        
        if unit_abbreviation == FuelEfficiencyUnits.MilePerUsGallon:
            return """mpg (U.S.)"""
        
        if unit_abbreviation == FuelEfficiencyUnits.MilePerUkGallon:
            return """mpg (imp.)"""
        
        if unit_abbreviation == FuelEfficiencyUnits.KilometerPerLiter:
            return """km/L"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class CapacitanceUnits(Enum):
        """
            CapacitanceUnits enumeration
        """
        
        Farad = 'Farad'
        """
            
        """
        
        Picofarad = 'Picofarad'
        """
            
        """
        
        Nanofarad = 'Nanofarad'
        """
            
        """
        
        Microfarad = 'Microfarad'
        """
            
        """
        
        Millifarad = 'Millifarad'
        """
            
        """
        
        Kilofarad = 'Kilofarad'
        """
            
        """
        
        Megafarad = 'Megafarad'
        """
            
        """
        

class CapacitanceDto:
    """
    A DTO representation of a Capacitance

    Attributes:
        value (float): The value of the Capacitance.
        unit (CapacitanceUnits): The specific unit that the Capacitance value is representing.
    """

    def __init__(self, value: float, unit: CapacitanceUnits):
        """
        Create a new DTO representation of a Capacitance

        Parameters:
            value (float): The value of the Capacitance.
            unit (CapacitanceUnits): The specific unit that the Capacitance value is representing.
        """
        self.value: float = value
        """
        The value of the Capacitance
        """
        self.unit: CapacitanceUnits = unit
        """
        The specific unit that the Capacitance value is representing
        """

    def to_json(self):
        """
        Get a Capacitance DTO JSON object representing the current unit.

        :return: JSON object represents Capacitance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Farad"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Capacitance DTO from a json representation.

        :param data: The Capacitance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Farad"}
        :return: A new instance of CapacitanceDto.
        :rtype: CapacitanceDto
        """
        return CapacitanceDto(value=data["value"], unit=CapacitanceUnits(data["unit"]))


class Capacitance(AbstractMeasure):
    """
    Capacitance is the ability of a body to store an electric charge.

    Args:
        value (float): The value.
        from_unit (CapacitanceUnits): The Capacitance unit to create from, The default unit is Farad
    """
    def __init__(self, value: float, from_unit: CapacitanceUnits = CapacitanceUnits.Farad):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__farads = None
        
        self.__picofarads = None
        
        self.__nanofarads = None
        
        self.__microfarads = None
        
        self.__millifarads = None
        
        self.__kilofarads = None
        
        self.__megafarads = None
        

    def convert(self, unit: CapacitanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: CapacitanceUnits = CapacitanceUnits.Farad) -> CapacitanceDto:
        """
        Get a new instance of Capacitance DTO representing the current unit.

        :param hold_in_unit: The specific Capacitance unit to store the Capacitance value in the DTO representation.
        :type hold_in_unit: CapacitanceUnits
        :return: A new instance of CapacitanceDto.
        :rtype: CapacitanceDto
        """
        return CapacitanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: CapacitanceUnits = CapacitanceUnits.Farad):
        """
        Get a Capacitance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Capacitance unit to store the Capacitance value in the DTO representation.
        :type hold_in_unit: CapacitanceUnits
        :return: JSON object represents Capacitance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Farad"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(capacitance_dto: CapacitanceDto):
        """
        Obtain a new instance of Capacitance from a DTO unit object.

        :param capacitance_dto: The Capacitance DTO representation.
        :type capacitance_dto: CapacitanceDto
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(capacitance_dto.value, capacitance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Capacitance from a DTO unit json representation.

        :param data: The Capacitance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Farad"}
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance.from_dto(CapacitanceDto.from_json(data))

    def __convert_from_base(self, from_unit: CapacitanceUnits) -> float:
        value = self._value
        
        if from_unit == CapacitanceUnits.Farad:
            return (value)
        
        if from_unit == CapacitanceUnits.Picofarad:
            return ((value) / 1e-12)
        
        if from_unit == CapacitanceUnits.Nanofarad:
            return ((value) / 1e-09)
        
        if from_unit == CapacitanceUnits.Microfarad:
            return ((value) / 1e-06)
        
        if from_unit == CapacitanceUnits.Millifarad:
            return ((value) / 0.001)
        
        if from_unit == CapacitanceUnits.Kilofarad:
            return ((value) / 1000.0)
        
        if from_unit == CapacitanceUnits.Megafarad:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: CapacitanceUnits) -> float:
        
        if to_unit == CapacitanceUnits.Farad:
            return (value)
        
        if to_unit == CapacitanceUnits.Picofarad:
            return ((value) * 1e-12)
        
        if to_unit == CapacitanceUnits.Nanofarad:
            return ((value) * 1e-09)
        
        if to_unit == CapacitanceUnits.Microfarad:
            return ((value) * 1e-06)
        
        if to_unit == CapacitanceUnits.Millifarad:
            return ((value) * 0.001)
        
        if to_unit == CapacitanceUnits.Kilofarad:
            return ((value) * 1000.0)
        
        if to_unit == CapacitanceUnits.Megafarad:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_farads(farads: float):
        """
        Create a new instance of Capacitance from a value in farads.

        

        :param meters: The Capacitance value in farads.
        :type farads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(farads, CapacitanceUnits.Farad)

    
    @staticmethod
    def from_picofarads(picofarads: float):
        """
        Create a new instance of Capacitance from a value in picofarads.

        

        :param meters: The Capacitance value in picofarads.
        :type picofarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(picofarads, CapacitanceUnits.Picofarad)

    
    @staticmethod
    def from_nanofarads(nanofarads: float):
        """
        Create a new instance of Capacitance from a value in nanofarads.

        

        :param meters: The Capacitance value in nanofarads.
        :type nanofarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(nanofarads, CapacitanceUnits.Nanofarad)

    
    @staticmethod
    def from_microfarads(microfarads: float):
        """
        Create a new instance of Capacitance from a value in microfarads.

        

        :param meters: The Capacitance value in microfarads.
        :type microfarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(microfarads, CapacitanceUnits.Microfarad)

    
    @staticmethod
    def from_millifarads(millifarads: float):
        """
        Create a new instance of Capacitance from a value in millifarads.

        

        :param meters: The Capacitance value in millifarads.
        :type millifarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(millifarads, CapacitanceUnits.Millifarad)

    
    @staticmethod
    def from_kilofarads(kilofarads: float):
        """
        Create a new instance of Capacitance from a value in kilofarads.

        

        :param meters: The Capacitance value in kilofarads.
        :type kilofarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(kilofarads, CapacitanceUnits.Kilofarad)

    
    @staticmethod
    def from_megafarads(megafarads: float):
        """
        Create a new instance of Capacitance from a value in megafarads.

        

        :param meters: The Capacitance value in megafarads.
        :type megafarads: float
        :return: A new instance of Capacitance.
        :rtype: Capacitance
        """
        return Capacitance(megafarads, CapacitanceUnits.Megafarad)

    
    @property
    def farads(self) -> float:
        """
        
        """
        if self.__farads != None:
            return self.__farads
        self.__farads = self.__convert_from_base(CapacitanceUnits.Farad)
        return self.__farads

    
    @property
    def picofarads(self) -> float:
        """
        
        """
        if self.__picofarads != None:
            return self.__picofarads
        self.__picofarads = self.__convert_from_base(CapacitanceUnits.Picofarad)
        return self.__picofarads

    
    @property
    def nanofarads(self) -> float:
        """
        
        """
        if self.__nanofarads != None:
            return self.__nanofarads
        self.__nanofarads = self.__convert_from_base(CapacitanceUnits.Nanofarad)
        return self.__nanofarads

    
    @property
    def microfarads(self) -> float:
        """
        
        """
        if self.__microfarads != None:
            return self.__microfarads
        self.__microfarads = self.__convert_from_base(CapacitanceUnits.Microfarad)
        return self.__microfarads

    
    @property
    def millifarads(self) -> float:
        """
        
        """
        if self.__millifarads != None:
            return self.__millifarads
        self.__millifarads = self.__convert_from_base(CapacitanceUnits.Millifarad)
        return self.__millifarads

    
    @property
    def kilofarads(self) -> float:
        """
        
        """
        if self.__kilofarads != None:
            return self.__kilofarads
        self.__kilofarads = self.__convert_from_base(CapacitanceUnits.Kilofarad)
        return self.__kilofarads

    
    @property
    def megafarads(self) -> float:
        """
        
        """
        if self.__megafarads != None:
            return self.__megafarads
        self.__megafarads = self.__convert_from_base(CapacitanceUnits.Megafarad)
        return self.__megafarads

    
    def to_string(self, unit: CapacitanceUnits = CapacitanceUnits.Farad, fractional_digits: int = None) -> str:
        """
        Format the Capacitance to a string.
        
        Note: the default format for Capacitance is Farad.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Capacitance. Default is 'Farad'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == CapacitanceUnits.Farad:
            return f"""{super()._truncate_fraction_digits(self.farads, fractional_digits)} F"""
        
        if unit == CapacitanceUnits.Picofarad:
            return f"""{super()._truncate_fraction_digits(self.picofarads, fractional_digits)} pF"""
        
        if unit == CapacitanceUnits.Nanofarad:
            return f"""{super()._truncate_fraction_digits(self.nanofarads, fractional_digits)} nF"""
        
        if unit == CapacitanceUnits.Microfarad:
            return f"""{super()._truncate_fraction_digits(self.microfarads, fractional_digits)} μF"""
        
        if unit == CapacitanceUnits.Millifarad:
            return f"""{super()._truncate_fraction_digits(self.millifarads, fractional_digits)} mF"""
        
        if unit == CapacitanceUnits.Kilofarad:
            return f"""{super()._truncate_fraction_digits(self.kilofarads, fractional_digits)} kF"""
        
        if unit == CapacitanceUnits.Megafarad:
            return f"""{super()._truncate_fraction_digits(self.megafarads, fractional_digits)} MF"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: CapacitanceUnits = CapacitanceUnits.Farad) -> str:
        """
        Get Capacitance unit abbreviation.
        Note! the default abbreviation for Capacitance is Farad.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == CapacitanceUnits.Farad:
            return """F"""
        
        if unit_abbreviation == CapacitanceUnits.Picofarad:
            return """pF"""
        
        if unit_abbreviation == CapacitanceUnits.Nanofarad:
            return """nF"""
        
        if unit_abbreviation == CapacitanceUnits.Microfarad:
            return """μF"""
        
        if unit_abbreviation == CapacitanceUnits.Millifarad:
            return """mF"""
        
        if unit_abbreviation == CapacitanceUnits.Kilofarad:
            return """kF"""
        
        if unit_abbreviation == CapacitanceUnits.Megafarad:
            return """MF"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class EnergyDensityUnits(Enum):
        """
            EnergyDensityUnits enumeration
        """
        
        JoulePerCubicMeter = 'JoulePerCubicMeter'
        """
            
        """
        
        WattHourPerCubicMeter = 'WattHourPerCubicMeter'
        """
            
        """
        
        KilojoulePerCubicMeter = 'KilojoulePerCubicMeter'
        """
            
        """
        
        MegajoulePerCubicMeter = 'MegajoulePerCubicMeter'
        """
            
        """
        
        GigajoulePerCubicMeter = 'GigajoulePerCubicMeter'
        """
            
        """
        
        TerajoulePerCubicMeter = 'TerajoulePerCubicMeter'
        """
            
        """
        
        PetajoulePerCubicMeter = 'PetajoulePerCubicMeter'
        """
            
        """
        
        KilowattHourPerCubicMeter = 'KilowattHourPerCubicMeter'
        """
            
        """
        
        MegawattHourPerCubicMeter = 'MegawattHourPerCubicMeter'
        """
            
        """
        
        GigawattHourPerCubicMeter = 'GigawattHourPerCubicMeter'
        """
            
        """
        
        TerawattHourPerCubicMeter = 'TerawattHourPerCubicMeter'
        """
            
        """
        
        PetawattHourPerCubicMeter = 'PetawattHourPerCubicMeter'
        """
            
        """
        

class EnergyDensityDto:
    """
    A DTO representation of a EnergyDensity

    Attributes:
        value (float): The value of the EnergyDensity.
        unit (EnergyDensityUnits): The specific unit that the EnergyDensity value is representing.
    """

    def __init__(self, value: float, unit: EnergyDensityUnits):
        """
        Create a new DTO representation of a EnergyDensity

        Parameters:
            value (float): The value of the EnergyDensity.
            unit (EnergyDensityUnits): The specific unit that the EnergyDensity value is representing.
        """
        self.value: float = value
        """
        The value of the EnergyDensity
        """
        self.unit: EnergyDensityUnits = unit
        """
        The specific unit that the EnergyDensity value is representing
        """

    def to_json(self):
        """
        Get a EnergyDensity DTO JSON object representing the current unit.

        :return: JSON object represents EnergyDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerCubicMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of EnergyDensity DTO from a json representation.

        :param data: The EnergyDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerCubicMeter"}
        :return: A new instance of EnergyDensityDto.
        :rtype: EnergyDensityDto
        """
        return EnergyDensityDto(value=data["value"], unit=EnergyDensityUnits(data["unit"]))


class EnergyDensity(AbstractMeasure):
    """
    None

    Args:
        value (float): The value.
        from_unit (EnergyDensityUnits): The EnergyDensity unit to create from, The default unit is JoulePerCubicMeter
    """
    def __init__(self, value: float, from_unit: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_cubic_meter = None
        
        self.__watt_hours_per_cubic_meter = None
        
        self.__kilojoules_per_cubic_meter = None
        
        self.__megajoules_per_cubic_meter = None
        
        self.__gigajoules_per_cubic_meter = None
        
        self.__terajoules_per_cubic_meter = None
        
        self.__petajoules_per_cubic_meter = None
        
        self.__kilowatt_hours_per_cubic_meter = None
        
        self.__megawatt_hours_per_cubic_meter = None
        
        self.__gigawatt_hours_per_cubic_meter = None
        
        self.__terawatt_hours_per_cubic_meter = None
        
        self.__petawatt_hours_per_cubic_meter = None
        

    def convert(self, unit: EnergyDensityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter) -> EnergyDensityDto:
        """
        Get a new instance of EnergyDensity DTO representing the current unit.

        :param hold_in_unit: The specific EnergyDensity unit to store the EnergyDensity value in the DTO representation.
        :type hold_in_unit: EnergyDensityUnits
        :return: A new instance of EnergyDensityDto.
        :rtype: EnergyDensityDto
        """
        return EnergyDensityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter):
        """
        Get a EnergyDensity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific EnergyDensity unit to store the EnergyDensity value in the DTO representation.
        :type hold_in_unit: EnergyDensityUnits
        :return: JSON object represents EnergyDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerCubicMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(energy_density_dto: EnergyDensityDto):
        """
        Obtain a new instance of EnergyDensity from a DTO unit object.

        :param energy_density_dto: The EnergyDensity DTO representation.
        :type energy_density_dto: EnergyDensityDto
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(energy_density_dto.value, energy_density_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of EnergyDensity from a DTO unit json representation.

        :param data: The EnergyDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerCubicMeter"}
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity.from_dto(EnergyDensityDto.from_json(data))

    def __convert_from_base(self, from_unit: EnergyDensityUnits) -> float:
        value = self._value
        
        if from_unit == EnergyDensityUnits.JoulePerCubicMeter:
            return (value)
        
        if from_unit == EnergyDensityUnits.WattHourPerCubicMeter:
            return (value / 3.6e+3)
        
        if from_unit == EnergyDensityUnits.KilojoulePerCubicMeter:
            return ((value) / 1000.0)
        
        if from_unit == EnergyDensityUnits.MegajoulePerCubicMeter:
            return ((value) / 1000000.0)
        
        if from_unit == EnergyDensityUnits.GigajoulePerCubicMeter:
            return ((value) / 1000000000.0)
        
        if from_unit == EnergyDensityUnits.TerajoulePerCubicMeter:
            return ((value) / 1000000000000.0)
        
        if from_unit == EnergyDensityUnits.PetajoulePerCubicMeter:
            return ((value) / 1000000000000000.0)
        
        if from_unit == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000.0)
        
        if from_unit == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000.0)
        
        if from_unit == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000000.0)
        
        if from_unit == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000000000.0)
        
        if from_unit == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return ((value / 3.6e+3) / 1000000000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: EnergyDensityUnits) -> float:
        
        if to_unit == EnergyDensityUnits.JoulePerCubicMeter:
            return (value)
        
        if to_unit == EnergyDensityUnits.WattHourPerCubicMeter:
            return (value * 3.6e+3)
        
        if to_unit == EnergyDensityUnits.KilojoulePerCubicMeter:
            return ((value) * 1000.0)
        
        if to_unit == EnergyDensityUnits.MegajoulePerCubicMeter:
            return ((value) * 1000000.0)
        
        if to_unit == EnergyDensityUnits.GigajoulePerCubicMeter:
            return ((value) * 1000000000.0)
        
        if to_unit == EnergyDensityUnits.TerajoulePerCubicMeter:
            return ((value) * 1000000000000.0)
        
        if to_unit == EnergyDensityUnits.PetajoulePerCubicMeter:
            return ((value) * 1000000000000000.0)
        
        if to_unit == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000.0)
        
        if to_unit == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000.0)
        
        if to_unit == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000000.0)
        
        if to_unit == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000000000.0)
        
        if to_unit == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return ((value * 3.6e+3) * 1000000000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_cubic_meter(joules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in joules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in joules_per_cubic_meter.
        :type joules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(joules_per_cubic_meter, EnergyDensityUnits.JoulePerCubicMeter)

    
    @staticmethod
    def from_watt_hours_per_cubic_meter(watt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in watt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in watt_hours_per_cubic_meter.
        :type watt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(watt_hours_per_cubic_meter, EnergyDensityUnits.WattHourPerCubicMeter)

    
    @staticmethod
    def from_kilojoules_per_cubic_meter(kilojoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in kilojoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in kilojoules_per_cubic_meter.
        :type kilojoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(kilojoules_per_cubic_meter, EnergyDensityUnits.KilojoulePerCubicMeter)

    
    @staticmethod
    def from_megajoules_per_cubic_meter(megajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in megajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in megajoules_per_cubic_meter.
        :type megajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(megajoules_per_cubic_meter, EnergyDensityUnits.MegajoulePerCubicMeter)

    
    @staticmethod
    def from_gigajoules_per_cubic_meter(gigajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in gigajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in gigajoules_per_cubic_meter.
        :type gigajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(gigajoules_per_cubic_meter, EnergyDensityUnits.GigajoulePerCubicMeter)

    
    @staticmethod
    def from_terajoules_per_cubic_meter(terajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in terajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in terajoules_per_cubic_meter.
        :type terajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(terajoules_per_cubic_meter, EnergyDensityUnits.TerajoulePerCubicMeter)

    
    @staticmethod
    def from_petajoules_per_cubic_meter(petajoules_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in petajoules_per_cubic_meter.

        

        :param meters: The EnergyDensity value in petajoules_per_cubic_meter.
        :type petajoules_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(petajoules_per_cubic_meter, EnergyDensityUnits.PetajoulePerCubicMeter)

    
    @staticmethod
    def from_kilowatt_hours_per_cubic_meter(kilowatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in kilowatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in kilowatt_hours_per_cubic_meter.
        :type kilowatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(kilowatt_hours_per_cubic_meter, EnergyDensityUnits.KilowattHourPerCubicMeter)

    
    @staticmethod
    def from_megawatt_hours_per_cubic_meter(megawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in megawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in megawatt_hours_per_cubic_meter.
        :type megawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(megawatt_hours_per_cubic_meter, EnergyDensityUnits.MegawattHourPerCubicMeter)

    
    @staticmethod
    def from_gigawatt_hours_per_cubic_meter(gigawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in gigawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in gigawatt_hours_per_cubic_meter.
        :type gigawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(gigawatt_hours_per_cubic_meter, EnergyDensityUnits.GigawattHourPerCubicMeter)

    
    @staticmethod
    def from_terawatt_hours_per_cubic_meter(terawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in terawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in terawatt_hours_per_cubic_meter.
        :type terawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(terawatt_hours_per_cubic_meter, EnergyDensityUnits.TerawattHourPerCubicMeter)

    
    @staticmethod
    def from_petawatt_hours_per_cubic_meter(petawatt_hours_per_cubic_meter: float):
        """
        Create a new instance of EnergyDensity from a value in petawatt_hours_per_cubic_meter.

        

        :param meters: The EnergyDensity value in petawatt_hours_per_cubic_meter.
        :type petawatt_hours_per_cubic_meter: float
        :return: A new instance of EnergyDensity.
        :rtype: EnergyDensity
        """
        return EnergyDensity(petawatt_hours_per_cubic_meter, EnergyDensityUnits.PetawattHourPerCubicMeter)

    
    @property
    def joules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__joules_per_cubic_meter != None:
            return self.__joules_per_cubic_meter
        self.__joules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.JoulePerCubicMeter)
        return self.__joules_per_cubic_meter

    
    @property
    def watt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__watt_hours_per_cubic_meter != None:
            return self.__watt_hours_per_cubic_meter
        self.__watt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.WattHourPerCubicMeter)
        return self.__watt_hours_per_cubic_meter

    
    @property
    def kilojoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__kilojoules_per_cubic_meter != None:
            return self.__kilojoules_per_cubic_meter
        self.__kilojoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.KilojoulePerCubicMeter)
        return self.__kilojoules_per_cubic_meter

    
    @property
    def megajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__megajoules_per_cubic_meter != None:
            return self.__megajoules_per_cubic_meter
        self.__megajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.MegajoulePerCubicMeter)
        return self.__megajoules_per_cubic_meter

    
    @property
    def gigajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__gigajoules_per_cubic_meter != None:
            return self.__gigajoules_per_cubic_meter
        self.__gigajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.GigajoulePerCubicMeter)
        return self.__gigajoules_per_cubic_meter

    
    @property
    def terajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__terajoules_per_cubic_meter != None:
            return self.__terajoules_per_cubic_meter
        self.__terajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.TerajoulePerCubicMeter)
        return self.__terajoules_per_cubic_meter

    
    @property
    def petajoules_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__petajoules_per_cubic_meter != None:
            return self.__petajoules_per_cubic_meter
        self.__petajoules_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.PetajoulePerCubicMeter)
        return self.__petajoules_per_cubic_meter

    
    @property
    def kilowatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__kilowatt_hours_per_cubic_meter != None:
            return self.__kilowatt_hours_per_cubic_meter
        self.__kilowatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.KilowattHourPerCubicMeter)
        return self.__kilowatt_hours_per_cubic_meter

    
    @property
    def megawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__megawatt_hours_per_cubic_meter != None:
            return self.__megawatt_hours_per_cubic_meter
        self.__megawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.MegawattHourPerCubicMeter)
        return self.__megawatt_hours_per_cubic_meter

    
    @property
    def gigawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__gigawatt_hours_per_cubic_meter != None:
            return self.__gigawatt_hours_per_cubic_meter
        self.__gigawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.GigawattHourPerCubicMeter)
        return self.__gigawatt_hours_per_cubic_meter

    
    @property
    def terawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__terawatt_hours_per_cubic_meter != None:
            return self.__terawatt_hours_per_cubic_meter
        self.__terawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.TerawattHourPerCubicMeter)
        return self.__terawatt_hours_per_cubic_meter

    
    @property
    def petawatt_hours_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__petawatt_hours_per_cubic_meter != None:
            return self.__petawatt_hours_per_cubic_meter
        self.__petawatt_hours_per_cubic_meter = self.__convert_from_base(EnergyDensityUnits.PetawattHourPerCubicMeter)
        return self.__petawatt_hours_per_cubic_meter

    
    def to_string(self, unit: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter, fractional_digits: int = None) -> str:
        """
        Format the EnergyDensity to a string.
        
        Note: the default format for EnergyDensity is JoulePerCubicMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the EnergyDensity. Default is 'JoulePerCubicMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == EnergyDensityUnits.JoulePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.joules_per_cubic_meter, fractional_digits)} J/m³"""
        
        if unit == EnergyDensityUnits.WattHourPerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.watt_hours_per_cubic_meter, fractional_digits)} Wh/m³"""
        
        if unit == EnergyDensityUnits.KilojoulePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_cubic_meter, fractional_digits)} kJ/m³"""
        
        if unit == EnergyDensityUnits.MegajoulePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_cubic_meter, fractional_digits)} MJ/m³"""
        
        if unit == EnergyDensityUnits.GigajoulePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.gigajoules_per_cubic_meter, fractional_digits)} GJ/m³"""
        
        if unit == EnergyDensityUnits.TerajoulePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.terajoules_per_cubic_meter, fractional_digits)} TJ/m³"""
        
        if unit == EnergyDensityUnits.PetajoulePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.petajoules_per_cubic_meter, fractional_digits)} PJ/m³"""
        
        if unit == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_hours_per_cubic_meter, fractional_digits)} kWh/m³"""
        
        if unit == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.megawatt_hours_per_cubic_meter, fractional_digits)} MWh/m³"""
        
        if unit == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.gigawatt_hours_per_cubic_meter, fractional_digits)} GWh/m³"""
        
        if unit == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.terawatt_hours_per_cubic_meter, fractional_digits)} TWh/m³"""
        
        if unit == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.petawatt_hours_per_cubic_meter, fractional_digits)} PWh/m³"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: EnergyDensityUnits = EnergyDensityUnits.JoulePerCubicMeter) -> str:
        """
        Get EnergyDensity unit abbreviation.
        Note! the default abbreviation for EnergyDensity is JoulePerCubicMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == EnergyDensityUnits.JoulePerCubicMeter:
            return """J/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.WattHourPerCubicMeter:
            return """Wh/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.KilojoulePerCubicMeter:
            return """kJ/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.MegajoulePerCubicMeter:
            return """MJ/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.GigajoulePerCubicMeter:
            return """GJ/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.TerajoulePerCubicMeter:
            return """TJ/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.PetajoulePerCubicMeter:
            return """PJ/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.KilowattHourPerCubicMeter:
            return """kWh/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.MegawattHourPerCubicMeter:
            return """MWh/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.GigawattHourPerCubicMeter:
            return """GWh/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.TerawattHourPerCubicMeter:
            return """TWh/m³"""
        
        if unit_abbreviation == EnergyDensityUnits.PetawattHourPerCubicMeter:
            return """PWh/m³"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class TemperatureUnits(Enum):
        """
            TemperatureUnits enumeration
        """
        
        Kelvin = 'Kelvin'
        """
            
        """
        
        DegreeCelsius = 'DegreeCelsius'
        """
            
        """
        
        MillidegreeCelsius = 'MillidegreeCelsius'
        """
            
        """
        
        DegreeDelisle = 'DegreeDelisle'
        """
            
        """
        
        DegreeFahrenheit = 'DegreeFahrenheit'
        """
            
        """
        
        DegreeNewton = 'DegreeNewton'
        """
            
        """
        
        DegreeRankine = 'DegreeRankine'
        """
            
        """
        
        DegreeReaumur = 'DegreeReaumur'
        """
            
        """
        
        DegreeRoemer = 'DegreeRoemer'
        """
            
        """
        
        SolarTemperature = 'SolarTemperature'
        """
            
        """
        

class TemperatureDto:
    """
    A DTO representation of a Temperature

    Attributes:
        value (float): The value of the Temperature.
        unit (TemperatureUnits): The specific unit that the Temperature value is representing.
    """

    def __init__(self, value: float, unit: TemperatureUnits):
        """
        Create a new DTO representation of a Temperature

        Parameters:
            value (float): The value of the Temperature.
            unit (TemperatureUnits): The specific unit that the Temperature value is representing.
        """
        self.value: float = value
        """
        The value of the Temperature
        """
        self.unit: TemperatureUnits = unit
        """
        The specific unit that the Temperature value is representing
        """

    def to_json(self):
        """
        Get a Temperature DTO JSON object representing the current unit.

        :return: JSON object represents Temperature DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Kelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Temperature DTO from a json representation.

        :param data: The Temperature DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Kelvin"}
        :return: A new instance of TemperatureDto.
        :rtype: TemperatureDto
        """
        return TemperatureDto(value=data["value"], unit=TemperatureUnits(data["unit"]))


class Temperature(AbstractMeasure):
    """
    A temperature is a numerical measure of hot or cold. Its measurement is by detection of heat radiation or particle velocity or kinetic energy, or by the bulk behavior of a thermometric material. It may be calibrated in any of various temperature scales, Celsius, Fahrenheit, Kelvin, etc. The fundamental physical definition of temperature is provided by thermodynamics.

    Args:
        value (float): The value.
        from_unit (TemperatureUnits): The Temperature unit to create from, The default unit is Kelvin
    """
    def __init__(self, value: float, from_unit: TemperatureUnits = TemperatureUnits.Kelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__kelvins = None
        
        self.__degrees_celsius = None
        
        self.__millidegrees_celsius = None
        
        self.__degrees_delisle = None
        
        self.__degrees_fahrenheit = None
        
        self.__degrees_newton = None
        
        self.__degrees_rankine = None
        
        self.__degrees_reaumur = None
        
        self.__degrees_roemer = None
        
        self.__solar_temperatures = None
        

    def convert(self, unit: TemperatureUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: TemperatureUnits = TemperatureUnits.Kelvin) -> TemperatureDto:
        """
        Get a new instance of Temperature DTO representing the current unit.

        :param hold_in_unit: The specific Temperature unit to store the Temperature value in the DTO representation.
        :type hold_in_unit: TemperatureUnits
        :return: A new instance of TemperatureDto.
        :rtype: TemperatureDto
        """
        return TemperatureDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: TemperatureUnits = TemperatureUnits.Kelvin):
        """
        Get a Temperature DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Temperature unit to store the Temperature value in the DTO representation.
        :type hold_in_unit: TemperatureUnits
        :return: JSON object represents Temperature DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Kelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(temperature_dto: TemperatureDto):
        """
        Obtain a new instance of Temperature from a DTO unit object.

        :param temperature_dto: The Temperature DTO representation.
        :type temperature_dto: TemperatureDto
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(temperature_dto.value, temperature_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Temperature from a DTO unit json representation.

        :param data: The Temperature DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Kelvin"}
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature.from_dto(TemperatureDto.from_json(data))

    def __convert_from_base(self, from_unit: TemperatureUnits) -> float:
        value = self._value
        
        if from_unit == TemperatureUnits.Kelvin:
            return (value)
        
        if from_unit == TemperatureUnits.DegreeCelsius:
            return (value - 273.15)
        
        if from_unit == TemperatureUnits.MillidegreeCelsius:
            return ((value - 273.15) * 1000)
        
        if from_unit == TemperatureUnits.DegreeDelisle:
            return ((value - 373.15) * -3 / 2)
        
        if from_unit == TemperatureUnits.DegreeFahrenheit:
            return ((value - 459.67 * 5 / 9) * 9 / 5)
        
        if from_unit == TemperatureUnits.DegreeNewton:
            return ((value - 273.15) * 33 / 100)
        
        if from_unit == TemperatureUnits.DegreeRankine:
            return (value * 9 / 5)
        
        if from_unit == TemperatureUnits.DegreeReaumur:
            return ((value - 273.15) * 4 / 5)
        
        if from_unit == TemperatureUnits.DegreeRoemer:
            return ((value - (273.15 - 7.5 * 40 / 21)) * 21 / 40)
        
        if from_unit == TemperatureUnits.SolarTemperature:
            return (value / 5778)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TemperatureUnits) -> float:
        
        if to_unit == TemperatureUnits.Kelvin:
            return (value)
        
        if to_unit == TemperatureUnits.DegreeCelsius:
            return (value + 273.15)
        
        if to_unit == TemperatureUnits.MillidegreeCelsius:
            return (value / 1000 + 273.15)
        
        if to_unit == TemperatureUnits.DegreeDelisle:
            return (value * -2 / 3 + 373.15)
        
        if to_unit == TemperatureUnits.DegreeFahrenheit:
            return (value * 5 / 9 + 459.67 * 5 / 9)
        
        if to_unit == TemperatureUnits.DegreeNewton:
            return (value * 100 / 33 + 273.15)
        
        if to_unit == TemperatureUnits.DegreeRankine:
            return (value * 5 / 9)
        
        if to_unit == TemperatureUnits.DegreeReaumur:
            return (value * 5 / 4 + 273.15)
        
        if to_unit == TemperatureUnits.DegreeRoemer:
            return (value * 40 / 21 + 273.15 - 7.5 * 40 / 21)
        
        if to_unit == TemperatureUnits.SolarTemperature:
            return (value * 5778)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_kelvins(kelvins: float):
        """
        Create a new instance of Temperature from a value in kelvins.

        

        :param meters: The Temperature value in kelvins.
        :type kelvins: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(kelvins, TemperatureUnits.Kelvin)

    
    @staticmethod
    def from_degrees_celsius(degrees_celsius: float):
        """
        Create a new instance of Temperature from a value in degrees_celsius.

        

        :param meters: The Temperature value in degrees_celsius.
        :type degrees_celsius: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(degrees_celsius, TemperatureUnits.DegreeCelsius)

    
    @staticmethod
    def from_millidegrees_celsius(millidegrees_celsius: float):
        """
        Create a new instance of Temperature from a value in millidegrees_celsius.

        

        :param meters: The Temperature value in millidegrees_celsius.
        :type millidegrees_celsius: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(millidegrees_celsius, TemperatureUnits.MillidegreeCelsius)

    
    @staticmethod
    def from_degrees_delisle(degrees_delisle: float):
        """
        Create a new instance of Temperature from a value in degrees_delisle.

        

        :param meters: The Temperature value in degrees_delisle.
        :type degrees_delisle: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(degrees_delisle, TemperatureUnits.DegreeDelisle)

    
    @staticmethod
    def from_degrees_fahrenheit(degrees_fahrenheit: float):
        """
        Create a new instance of Temperature from a value in degrees_fahrenheit.

        

        :param meters: The Temperature value in degrees_fahrenheit.
        :type degrees_fahrenheit: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(degrees_fahrenheit, TemperatureUnits.DegreeFahrenheit)

    
    @staticmethod
    def from_degrees_newton(degrees_newton: float):
        """
        Create a new instance of Temperature from a value in degrees_newton.

        

        :param meters: The Temperature value in degrees_newton.
        :type degrees_newton: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(degrees_newton, TemperatureUnits.DegreeNewton)

    
    @staticmethod
    def from_degrees_rankine(degrees_rankine: float):
        """
        Create a new instance of Temperature from a value in degrees_rankine.

        

        :param meters: The Temperature value in degrees_rankine.
        :type degrees_rankine: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(degrees_rankine, TemperatureUnits.DegreeRankine)

    
    @staticmethod
    def from_degrees_reaumur(degrees_reaumur: float):
        """
        Create a new instance of Temperature from a value in degrees_reaumur.

        

        :param meters: The Temperature value in degrees_reaumur.
        :type degrees_reaumur: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(degrees_reaumur, TemperatureUnits.DegreeReaumur)

    
    @staticmethod
    def from_degrees_roemer(degrees_roemer: float):
        """
        Create a new instance of Temperature from a value in degrees_roemer.

        

        :param meters: The Temperature value in degrees_roemer.
        :type degrees_roemer: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(degrees_roemer, TemperatureUnits.DegreeRoemer)

    
    @staticmethod
    def from_solar_temperatures(solar_temperatures: float):
        """
        Create a new instance of Temperature from a value in solar_temperatures.

        

        :param meters: The Temperature value in solar_temperatures.
        :type solar_temperatures: float
        :return: A new instance of Temperature.
        :rtype: Temperature
        """
        return Temperature(solar_temperatures, TemperatureUnits.SolarTemperature)

    
    @property
    def kelvins(self) -> float:
        """
        
        """
        if self.__kelvins != None:
            return self.__kelvins
        self.__kelvins = self.__convert_from_base(TemperatureUnits.Kelvin)
        return self.__kelvins

    
    @property
    def degrees_celsius(self) -> float:
        """
        
        """
        if self.__degrees_celsius != None:
            return self.__degrees_celsius
        self.__degrees_celsius = self.__convert_from_base(TemperatureUnits.DegreeCelsius)
        return self.__degrees_celsius

    
    @property
    def millidegrees_celsius(self) -> float:
        """
        
        """
        if self.__millidegrees_celsius != None:
            return self.__millidegrees_celsius
        self.__millidegrees_celsius = self.__convert_from_base(TemperatureUnits.MillidegreeCelsius)
        return self.__millidegrees_celsius

    
    @property
    def degrees_delisle(self) -> float:
        """
        
        """
        if self.__degrees_delisle != None:
            return self.__degrees_delisle
        self.__degrees_delisle = self.__convert_from_base(TemperatureUnits.DegreeDelisle)
        return self.__degrees_delisle

    
    @property
    def degrees_fahrenheit(self) -> float:
        """
        
        """
        if self.__degrees_fahrenheit != None:
            return self.__degrees_fahrenheit
        self.__degrees_fahrenheit = self.__convert_from_base(TemperatureUnits.DegreeFahrenheit)
        return self.__degrees_fahrenheit

    
    @property
    def degrees_newton(self) -> float:
        """
        
        """
        if self.__degrees_newton != None:
            return self.__degrees_newton
        self.__degrees_newton = self.__convert_from_base(TemperatureUnits.DegreeNewton)
        return self.__degrees_newton

    
    @property
    def degrees_rankine(self) -> float:
        """
        
        """
        if self.__degrees_rankine != None:
            return self.__degrees_rankine
        self.__degrees_rankine = self.__convert_from_base(TemperatureUnits.DegreeRankine)
        return self.__degrees_rankine

    
    @property
    def degrees_reaumur(self) -> float:
        """
        
        """
        if self.__degrees_reaumur != None:
            return self.__degrees_reaumur
        self.__degrees_reaumur = self.__convert_from_base(TemperatureUnits.DegreeReaumur)
        return self.__degrees_reaumur

    
    @property
    def degrees_roemer(self) -> float:
        """
        
        """
        if self.__degrees_roemer != None:
            return self.__degrees_roemer
        self.__degrees_roemer = self.__convert_from_base(TemperatureUnits.DegreeRoemer)
        return self.__degrees_roemer

    
    @property
    def solar_temperatures(self) -> float:
        """
        
        """
        if self.__solar_temperatures != None:
            return self.__solar_temperatures
        self.__solar_temperatures = self.__convert_from_base(TemperatureUnits.SolarTemperature)
        return self.__solar_temperatures

    
    def to_string(self, unit: TemperatureUnits = TemperatureUnits.Kelvin, fractional_digits: int = None) -> str:
        """
        Format the Temperature to a string.
        
        Note: the default format for Temperature is Kelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Temperature. Default is 'Kelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == TemperatureUnits.Kelvin:
            return f"""{super()._truncate_fraction_digits(self.kelvins, fractional_digits)} K"""
        
        if unit == TemperatureUnits.DegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.degrees_celsius, fractional_digits)} °C"""
        
        if unit == TemperatureUnits.MillidegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.millidegrees_celsius, fractional_digits)} m°C"""
        
        if unit == TemperatureUnits.DegreeDelisle:
            return f"""{super()._truncate_fraction_digits(self.degrees_delisle, fractional_digits)} °De"""
        
        if unit == TemperatureUnits.DegreeFahrenheit:
            return f"""{super()._truncate_fraction_digits(self.degrees_fahrenheit, fractional_digits)} °F"""
        
        if unit == TemperatureUnits.DegreeNewton:
            return f"""{super()._truncate_fraction_digits(self.degrees_newton, fractional_digits)} °N"""
        
        if unit == TemperatureUnits.DegreeRankine:
            return f"""{super()._truncate_fraction_digits(self.degrees_rankine, fractional_digits)} °R"""
        
        if unit == TemperatureUnits.DegreeReaumur:
            return f"""{super()._truncate_fraction_digits(self.degrees_reaumur, fractional_digits)} °Ré"""
        
        if unit == TemperatureUnits.DegreeRoemer:
            return f"""{super()._truncate_fraction_digits(self.degrees_roemer, fractional_digits)} °Rø"""
        
        if unit == TemperatureUnits.SolarTemperature:
            return f"""{super()._truncate_fraction_digits(self.solar_temperatures, fractional_digits)} T⊙"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: TemperatureUnits = TemperatureUnits.Kelvin) -> str:
        """
        Get Temperature unit abbreviation.
        Note! the default abbreviation for Temperature is Kelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TemperatureUnits.Kelvin:
            return """K"""
        
        if unit_abbreviation == TemperatureUnits.DegreeCelsius:
            return """°C"""
        
        if unit_abbreviation == TemperatureUnits.MillidegreeCelsius:
            return """m°C"""
        
        if unit_abbreviation == TemperatureUnits.DegreeDelisle:
            return """°De"""
        
        if unit_abbreviation == TemperatureUnits.DegreeFahrenheit:
            return """°F"""
        
        if unit_abbreviation == TemperatureUnits.DegreeNewton:
            return """°N"""
        
        if unit_abbreviation == TemperatureUnits.DegreeRankine:
            return """°R"""
        
        if unit_abbreviation == TemperatureUnits.DegreeReaumur:
            return """°Ré"""
        
        if unit_abbreviation == TemperatureUnits.DegreeRoemer:
            return """°Rø"""
        
        if unit_abbreviation == TemperatureUnits.SolarTemperature:
            return """T⊙"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ReactiveEnergyUnits(Enum):
        """
            ReactiveEnergyUnits enumeration
        """
        
        VoltampereReactiveHour = 'VoltampereReactiveHour'
        """
            
        """
        
        KilovoltampereReactiveHour = 'KilovoltampereReactiveHour'
        """
            
        """
        
        MegavoltampereReactiveHour = 'MegavoltampereReactiveHour'
        """
            
        """
        

class ReactiveEnergyDto:
    """
    A DTO representation of a ReactiveEnergy

    Attributes:
        value (float): The value of the ReactiveEnergy.
        unit (ReactiveEnergyUnits): The specific unit that the ReactiveEnergy value is representing.
    """

    def __init__(self, value: float, unit: ReactiveEnergyUnits):
        """
        Create a new DTO representation of a ReactiveEnergy

        Parameters:
            value (float): The value of the ReactiveEnergy.
            unit (ReactiveEnergyUnits): The specific unit that the ReactiveEnergy value is representing.
        """
        self.value: float = value
        """
        The value of the ReactiveEnergy
        """
        self.unit: ReactiveEnergyUnits = unit
        """
        The specific unit that the ReactiveEnergy value is representing
        """

    def to_json(self):
        """
        Get a ReactiveEnergy DTO JSON object representing the current unit.

        :return: JSON object represents ReactiveEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltampereReactiveHour"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ReactiveEnergy DTO from a json representation.

        :param data: The ReactiveEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltampereReactiveHour"}
        :return: A new instance of ReactiveEnergyDto.
        :rtype: ReactiveEnergyDto
        """
        return ReactiveEnergyDto(value=data["value"], unit=ReactiveEnergyUnits(data["unit"]))


class ReactiveEnergy(AbstractMeasure):
    """
    The Volt-ampere reactive hour (expressed as varh) is the reactive power of one Volt-ampere reactive produced in one hour.

    Args:
        value (float): The value.
        from_unit (ReactiveEnergyUnits): The ReactiveEnergy unit to create from, The default unit is VoltampereReactiveHour
    """
    def __init__(self, value: float, from_unit: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__voltampere_reactive_hours = None
        
        self.__kilovoltampere_reactive_hours = None
        
        self.__megavoltampere_reactive_hours = None
        

    def convert(self, unit: ReactiveEnergyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour) -> ReactiveEnergyDto:
        """
        Get a new instance of ReactiveEnergy DTO representing the current unit.

        :param hold_in_unit: The specific ReactiveEnergy unit to store the ReactiveEnergy value in the DTO representation.
        :type hold_in_unit: ReactiveEnergyUnits
        :return: A new instance of ReactiveEnergyDto.
        :rtype: ReactiveEnergyDto
        """
        return ReactiveEnergyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour):
        """
        Get a ReactiveEnergy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ReactiveEnergy unit to store the ReactiveEnergy value in the DTO representation.
        :type hold_in_unit: ReactiveEnergyUnits
        :return: JSON object represents ReactiveEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltampereReactiveHour"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(reactive_energy_dto: ReactiveEnergyDto):
        """
        Obtain a new instance of ReactiveEnergy from a DTO unit object.

        :param reactive_energy_dto: The ReactiveEnergy DTO representation.
        :type reactive_energy_dto: ReactiveEnergyDto
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy(reactive_energy_dto.value, reactive_energy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ReactiveEnergy from a DTO unit json representation.

        :param data: The ReactiveEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltampereReactiveHour"}
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy.from_dto(ReactiveEnergyDto.from_json(data))

    def __convert_from_base(self, from_unit: ReactiveEnergyUnits) -> float:
        value = self._value
        
        if from_unit == ReactiveEnergyUnits.VoltampereReactiveHour:
            return (value)
        
        if from_unit == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return ((value) / 1000.0)
        
        if from_unit == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ReactiveEnergyUnits) -> float:
        
        if to_unit == ReactiveEnergyUnits.VoltampereReactiveHour:
            return (value)
        
        if to_unit == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return ((value) * 1000.0)
        
        if to_unit == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_voltampere_reactive_hours(voltampere_reactive_hours: float):
        """
        Create a new instance of ReactiveEnergy from a value in voltampere_reactive_hours.

        

        :param meters: The ReactiveEnergy value in voltampere_reactive_hours.
        :type voltampere_reactive_hours: float
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy(voltampere_reactive_hours, ReactiveEnergyUnits.VoltampereReactiveHour)

    
    @staticmethod
    def from_kilovoltampere_reactive_hours(kilovoltampere_reactive_hours: float):
        """
        Create a new instance of ReactiveEnergy from a value in kilovoltampere_reactive_hours.

        

        :param meters: The ReactiveEnergy value in kilovoltampere_reactive_hours.
        :type kilovoltampere_reactive_hours: float
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy(kilovoltampere_reactive_hours, ReactiveEnergyUnits.KilovoltampereReactiveHour)

    
    @staticmethod
    def from_megavoltampere_reactive_hours(megavoltampere_reactive_hours: float):
        """
        Create a new instance of ReactiveEnergy from a value in megavoltampere_reactive_hours.

        

        :param meters: The ReactiveEnergy value in megavoltampere_reactive_hours.
        :type megavoltampere_reactive_hours: float
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy(megavoltampere_reactive_hours, ReactiveEnergyUnits.MegavoltampereReactiveHour)

    
    @property
    def voltampere_reactive_hours(self) -> float:
        """
        
        """
        if self.__voltampere_reactive_hours != None:
            return self.__voltampere_reactive_hours
        self.__voltampere_reactive_hours = self.__convert_from_base(ReactiveEnergyUnits.VoltampereReactiveHour)
        return self.__voltampere_reactive_hours

    
    @property
    def kilovoltampere_reactive_hours(self) -> float:
        """
        
        """
        if self.__kilovoltampere_reactive_hours != None:
            return self.__kilovoltampere_reactive_hours
        self.__kilovoltampere_reactive_hours = self.__convert_from_base(ReactiveEnergyUnits.KilovoltampereReactiveHour)
        return self.__kilovoltampere_reactive_hours

    
    @property
    def megavoltampere_reactive_hours(self) -> float:
        """
        
        """
        if self.__megavoltampere_reactive_hours != None:
            return self.__megavoltampere_reactive_hours
        self.__megavoltampere_reactive_hours = self.__convert_from_base(ReactiveEnergyUnits.MegavoltampereReactiveHour)
        return self.__megavoltampere_reactive_hours

    
    def to_string(self, unit: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour, fractional_digits: int = None) -> str:
        """
        Format the ReactiveEnergy to a string.
        
        Note: the default format for ReactiveEnergy is VoltampereReactiveHour.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ReactiveEnergy. Default is 'VoltampereReactiveHour'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ReactiveEnergyUnits.VoltampereReactiveHour:
            return f"""{super()._truncate_fraction_digits(self.voltampere_reactive_hours, fractional_digits)} varh"""
        
        if unit == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return f"""{super()._truncate_fraction_digits(self.kilovoltampere_reactive_hours, fractional_digits)} kvarh"""
        
        if unit == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return f"""{super()._truncate_fraction_digits(self.megavoltampere_reactive_hours, fractional_digits)} Mvarh"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour) -> str:
        """
        Get ReactiveEnergy unit abbreviation.
        Note! the default abbreviation for ReactiveEnergy is VoltampereReactiveHour.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ReactiveEnergyUnits.VoltampereReactiveHour:
            return """varh"""
        
        if unit_abbreviation == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return """kvarh"""
        
        if unit_abbreviation == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return """Mvarh"""
        